@@ -1,5 +1,5 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
- * JFlex 1.7.0                                                             *
+ * JFlex 1.8.2                                                             *
  * Copyright (C) 1998-2018  Gerwin Klein <lsf@jflex.de>                    *
  * All rights reserved.                                                    *
  *                                                                         *
@@ -10,262 +10,180 @@
 package jflex;
 
 import java.io.File;
-import java.io.FileInputStream;
-import java.io.FileNotFoundException;
-import java.io.IOException;
-import java.io.InputStreamReader;
-import java.io.Reader;
 import java.lang.reflect.Field;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 import java.util.SortedMap;
 import java.util.SortedSet;
 import java.util.TreeMap;
 import java.util.TreeSet;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
+import jflex.base.Build;
+import jflex.core.OptionUtils;
+import jflex.core.unicode.UnicodeProperties;
+import jflex.exceptions.GeneratorException;
+import jflex.exceptions.SilentExit;
+import jflex.generator.LexGenerator;
 import jflex.gui.MainFrame;
-import jflex.unicode.UnicodeProperties;
+import jflex.l10n.ErrorMessages;
+import jflex.logging.Out;
+import jflex.option.Options;
 
 /**
- * This is the main class of JFlex controlling the scanner generation process. It is responsible for
- * parsing the commandline, getting input files, starting up the GUI if necessary, etc.
+ * This is the command-line interface.
+ *
+ * <p>It is responsible for parsing the commandline, getting input files, starting up the GUI if
+ * necessary, etc. and invokes {@link LexGenerator} accordingly.
  *
  * @author Gerwin Klein
- * @version JFlex 1.7.0
+ * @author Régis Décamps
+ * @version JFlex 1.8.2
  */
 public class Main {
 
-  /** JFlex version */
-  public static final String version = "1.7.0"; // $NON-NLS-1$
-
-  /**
-   * Generates a scanner for the specified input file.
-   *
-   * @param inputFile a file containing a lexical specification to generate a scanner for.
-   */
-  public static void generate(File inputFile) {
-
-    Out.resetCounters();
-
-    Timer totalTime = new Timer();
-    Timer time = new Timer();
-
-    LexScan scanner = null;
-    LexParse parser = null;
-    Reader inputReader = null;
-
-    totalTime.start();
-
-    try {
-      Out.println(ErrorMessages.READING, inputFile.toString());
-      inputReader = new InputStreamReader(new FileInputStream(inputFile), Options.encoding);
-      scanner = new LexScan(inputReader);
-      scanner.setFile(inputFile);
-      parser = new LexParse(scanner);
-    } catch (FileNotFoundException e) {
-      Out.error(ErrorMessages.CANNOT_OPEN, inputFile.toString());
-      throw new GeneratorException();
-    }
-
-    try {
-      NFA nfa = (NFA) parser.parse().value;
-
-      Out.checkErrors();
-
-      if (Options.dump) Out.dump(ErrorMessages.get(ErrorMessages.NFA_IS) + Out.NL + nfa + Out.NL);
-
-      if (Options.dot) nfa.writeDot(Emitter.normalize("nfa.dot", null)); // $NON-NLS-1$
-
-      Out.println(ErrorMessages.NFA_STATES, nfa.numStates);
-
-      time.start();
-      DFA dfa = nfa.getDFA();
-      time.stop();
-      Out.time(ErrorMessages.DFA_TOOK, time);
-
-      dfa.checkActions(scanner, parser);
-
-      nfa = null;
-
-      if (Options.dump) Out.dump(ErrorMessages.get(ErrorMessages.DFA_IS) + Out.NL + dfa + Out.NL);
-
-      if (Options.dot) dfa.writeDot(Emitter.normalize("dfa-big.dot", null)); // $NON-NLS-1$
-
-      Out.checkErrors();
-
-      time.start();
-      dfa.minimize();
-      time.stop();
-
-      Out.time(ErrorMessages.MIN_TOOK, time);
-
-      if (Options.dump) Out.dump(ErrorMessages.get(ErrorMessages.MIN_DFA_IS) + Out.NL + dfa);
-
-      if (Options.dot) dfa.writeDot(Emitter.normalize("dfa-min.dot", null)); // $NON-NLS-1$
-
-      time.start();
-
-      Emitter e = new Emitter(inputFile, parser, dfa);
-      e.emit();
-
-      time.stop();
-
-      Out.time(ErrorMessages.WRITE_TOOK, time);
-
-      totalTime.stop();
-
-      Out.time(ErrorMessages.TOTAL_TIME, totalTime);
-    } catch (ScannerException e) {
-      Out.error(e.file, e.message, e.line, e.column);
-      throw new GeneratorException();
-    } catch (MacroException e) {
-      Out.error(e.getMessage());
-      throw new GeneratorException();
-    } catch (IOException e) {
-      Out.error(ErrorMessages.IO_ERROR, e.toString());
-      throw new GeneratorException();
-    } catch (OutOfMemoryError e) {
-      Out.error(ErrorMessages.OUT_OF_MEMORY);
-      throw new GeneratorException();
-    } catch (GeneratorException e) {
-      throw new GeneratorException();
-    } catch (Exception e) {
-      e.printStackTrace();
-      throw new GeneratorException();
-    }
-  }
-
   /**
    * parseOptions.
    *
    * @param argv an array of {@link java.lang.String} objects.
    * @return a {@link java.util.List} object.
-   * @throws jflex.SilentExit if any.
+   * @throws SilentExit if any.
    */
-  public static List<File> parseOptions(String argv[]) throws SilentExit {
+  private static List<File> parseOptions(String[] argv) throws SilentExit {
     List<File> files = new ArrayList<>();
 
     for (int i = 0; i < argv.length; i++) {
 
-      if (argv[i].equals("-d") || argv[i].equals("--outdir")) { // $NON-NLS-1$ //$NON-NLS-2$
+      if (Objects.equals(argv[i], "-d")
+          || Objects.equals(argv[i], "--outdir")) { // $NON-NLS-1$ //$NON-NLS-2$
         if (++i >= argv.length) {
           Out.error(ErrorMessages.NO_DIRECTORY);
           throw new GeneratorException();
         }
-        Options.setDir(argv[i]);
+        OptionUtils.setDir(argv[i]);
         continue;
       }
 
-      if (argv[i].equals("--skel") || argv[i].equals("-skel")) { // $NON-NLS-1$ //$NON-NLS-2$
+      if (Objects.equals(argv[i], "--skel")
+          || Objects.equals(argv[i], "-skel")) { // $NON-NLS-1$ //$NON-NLS-2$
         if (++i >= argv.length) {
           Out.error(ErrorMessages.NO_SKEL_FILE);
           throw new GeneratorException();
         }
 
-        Options.setSkeleton(new File(argv[i]));
+        OptionUtils.setSkeleton(new File(argv[i]));
         continue;
       }
 
-      if (argv[i].equals("--encoding")) {
+      if (Objects.equals(argv[i], "--encoding")) {
         if (++i >= argv.length) {
           Out.error(ErrorMessages.NO_ENCODING);
           throw new GeneratorException();
         }
 
-        Options.setEncoding(argv[i]);
+        OptionUtils.setEncoding(argv[i]);
         continue;
       }
 
-      if (argv[i].equals("-jlex") || argv[i].equals("--jlex")) { // $NON-NLS-1$ //$NON-NLS-2$
+      if (Objects.equals(argv[i], "-jlex")
+          || Objects.equals(argv[i], "--jlex")) { // $NON-NLS-1$ //$NON-NLS-2$
         Options.jlex = true;
         continue;
       }
 
-      if (argv[i].equals("-v")
-          || argv[i].equals("--verbose")
-          || argv[i].equals("-verbose")) { // $NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
+      if (Objects.equals(argv[i], "-v")
+          || Objects.equals(argv[i], "--verbose")
+          || Objects.equals(argv[i], "-verbose")) { // $NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
         Options.verbose = true;
         Options.progress = true;
         Options.unused_warning = true;
         continue;
       }
 
-      if (argv[i].equals("-q")
-          || argv[i].equals("--quiet")
-          || argv[i].equals("-quiet")) { // $NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
+      if (Objects.equals(argv[i], "-q")
+          || Objects.equals(argv[i], "--quiet")
+          || Objects.equals(argv[i], "-quiet")) { // $NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
         Options.verbose = false;
         Options.progress = false;
         Options.unused_warning = false;
         continue;
       }
 
-      if (argv[i].equals("--warn-unused")) { // $NON-NLS-1$
+      if (Objects.equals(argv[i], "--warn-unused")) { // $NON-NLS-1$
         Options.unused_warning = true;
         continue;
       }
 
-      if (argv[i].equals("--no-warn-unused")) { // $NON-NLS-1$
+      if (Objects.equals(argv[i], "--no-warn-unused")) { // $NON-NLS-1$
         Options.unused_warning = false;
         continue;
       }
 
-      if (argv[i].equals("--dump") || argv[i].equals("-dump")) { // $NON-NLS-1$ //$NON-NLS-2$
+      if (Objects.equals(argv[i], "--dump")
+          || Objects.equals(argv[i], "-dump")) { // $NON-NLS-1$ //$NON-NLS-2$
         Options.dump = true;
         continue;
       }
 
-      if (argv[i].equals("--time") || argv[i].equals("-time")) { // $NON-NLS-1$ //$NON-NLS-2$
+      if (Objects.equals(argv[i], "--time")
+          || Objects.equals(argv[i], "-time")) { // $NON-NLS-1$ //$NON-NLS-2$
         Options.time = true;
         continue;
       }
 
-      if (argv[i].equals("--version") || argv[i].equals("-version")) { // $NON-NLS-1$ //$NON-NLS-2$
-        Out.println(ErrorMessages.THIS_IS_JFLEX, version);
+      if (Objects.equals(argv[i], "--version")
+          || Objects.equals(argv[i], "-version")) { // $NON-NLS-1$ //$NON-NLS-2$
+        Out.println(ErrorMessages.THIS_IS_JFLEX, Build.VERSION);
         throw new SilentExit(0);
       }
 
-      if (argv[i].equals("--dot") || argv[i].equals("-dot")) { // $NON-NLS-1$ //$NON-NLS-2$
+      if (Objects.equals(argv[i], "--dot")
+          || Objects.equals(argv[i], "-dot")) { // $NON-NLS-1$ //$NON-NLS-2$
         Options.dot = true;
         continue;
       }
 
-      if (argv[i].equals("--help")
-          || argv[i].equals("-h")
-          || argv[i].equals("/h")) { // $NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
+      if (Objects.equals(argv[i], "--help")
+          || Objects.equals(argv[i], "-h")
+          || Objects.equals(argv[i], "/h")) { // $NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
         printUsage();
         throw new SilentExit(0);
       }
 
-      if (argv[i].equals("--info") || argv[i].equals("-info")) { // $NON-NLS-1$ //$NON-NLS-2$
-        Out.printSystemInfo();
+      if (Objects.equals(argv[i], "--info")
+          || Objects.equals(argv[i], "-info")) { // $NON-NLS-1$ //$NON-NLS-2$
+        printSystemInfo();
         throw new SilentExit(0);
       }
 
-      if (argv[i].equals("--nomin") || argv[i].equals("-nomin")) { // $NON-NLS-1$ //$NON-NLS-2$
+      if (Objects.equals(argv[i], "--nomin")
+          || Objects.equals(argv[i], "-nomin")) { // $NON-NLS-1$ //$NON-NLS-2$
         Options.no_minimize = true;
         continue;
       }
 
-      if (argv[i].equals("--pack") || argv[i].equals("-pack")) { // $NON-NLS-1$ //$NON-NLS-2$
+      if (Objects.equals(argv[i], "--pack")
+          || Objects.equals(argv[i], "-pack")) { // $NON-NLS-1$ //$NON-NLS-2$
         /* no-op - pack is the only generation method */
         continue;
       }
 
-      if (argv[i].equals("--nobak") || argv[i].equals("-nobak")) { // $NON-NLS-1$ //$NON-NLS-2$
+      if (Objects.equals(argv[i], "--nobak")
+          || Objects.equals(argv[i], "-nobak")) { // $NON-NLS-1$ //$NON-NLS-2$
         Options.no_backup = true;
         continue;
       }
 
-      if (argv[i].equals("--legacydot")
-          || argv[i].equals("-legacydot")) { // $NON-NLS-1$ //$NON-NLS-2$
+      if (Objects.equals(argv[i], "--legacydot")
+          || Objects.equals(argv[i], "-legacydot")) { // $NON-NLS-1$ //$NON-NLS-2$
         Options.legacy_dot = true;
         continue;
       }
 
-      if (argv[i].equals("--uniprops")
-          || argv[i].equals("-uniprops")) { // $NON-NLS-1$ //$NON-NLS-2$
+      if (Objects.equals(argv[i], "--uniprops")
+          || Objects.equals(argv[i], "-uniprops")) { // $NON-NLS-1$ //$NON-NLS-2$
         if (++i >= argv.length) {
           Out.error(
               ErrorMessages.PROPS_ARG_REQUIRES_UNICODE_VERSION, UnicodeProperties.UNICODE_VERSIONS);
@@ -278,7 +196,7 @@ public class Main {
           Out.error(
               ErrorMessages.UNSUPPORTED_UNICODE_VERSION_SUPPORTED_ARE,
               UnicodeProperties.UNICODE_VERSIONS);
-          throw new GeneratorException();
+          throw new GeneratorException(e);
         }
         throw new SilentExit();
       }
@@ -327,13 +245,13 @@ public class Main {
       field = clazz.getField("propertyValueAliases");
       propertyValueAliases = (String[]) field.get(null);
     } catch (Exception e) {
-      throw new UnicodeProperties.UnsupportedUnicodeVersionException();
+      throw new UnicodeProperties.UnsupportedUnicodeVersionException(e);
     }
     SortedMap<String, SortedSet<String>> propertyValuesToAliases = new TreeMap<>();
     for (String value : propertyValues) {
       propertyValuesToAliases.put(value, new TreeSet<String>());
     }
-    for (int i = 0; i < propertyValueAliases.length; i += 2) {
+    for (int i = 0; i < propertyValueAliases.length - 1; i += 2) {
       String alias = propertyValueAliases[i];
       String value = propertyValueAliases[i + 1];
       SortedSet<String> aliases = propertyValuesToAliases.get(value);
@@ -357,7 +275,7 @@ public class Main {
   }
 
   /** Prints the cli usage on stdout. */
-  public static void printUsage() {
+  private static void printUsage() {
     Out.println(""); // $NON-NLS-1$
     Out.println("Usage: jflex <options> <input-files>");
     Out.println("");
@@ -384,7 +302,7 @@ public class Main {
     Out.println("--help");
     Out.println("-h                 print this message");
     Out.println("");
-    Out.println(ErrorMessages.THIS_IS_JFLEX, version);
+    Out.println(ErrorMessages.THIS_IS_JFLEX, Build.VERSION);
     Out.println("Have a nice day!");
   }
 
@@ -392,32 +310,69 @@ public class Main {
    * generate.
    *
    * @param argv an array of {@link java.lang.String} objects.
-   * @throws jflex.SilentExit if any.
+   * @throws SilentExit if any.
    */
-  public static void generate(String argv[]) throws SilentExit {
+  public static void generate(String[] argv) throws SilentExit {
     List<File> files = parseOptions(argv);
 
     if (files.size() > 0) {
-      for (File file : files) generate(file);
+      for (File file : files) {
+        new LexGenerator(file).generate();
+      }
     } else {
       new MainFrame();
     }
   }
 
   /**
-   * Starts the generation process with the files in <code>argv</code> or pops up a window to choose
-   * a file, when <code>argv</code> doesn't have any file entries.
+   * Starts the generation process with the files in {@code argv} or pops up a window to choose a
+   * file, when {@code argv} doesn't have any file entries.
    *
    * @param argv the commandline.
    */
-  public static void main(String argv[]) {
+  public static void main(String[] argv) {
+    OptionUtils.setDefaultOptions();
     try {
       generate(argv);
     } catch (GeneratorException e) {
-      Out.statistics();
+      if (e.isUnExpected()) {
+        Out.error(
+            "Unexpected exception encountered. This indicates a bug in JFlex."
+                + Out.NL
+                + "Please consider filing an issue at http://github.com/jflex-de/jflex/issues/new"
+                + Out.NL);
+        Throwable cause = e.getCause();
+        if (cause != null) {
+          String msg = cause.getLocalizedMessage();
+          if (msg != null) Out.error(msg);
+          cause.printStackTrace();
+        }
+      } else {
+        Out.statistics();
+      }
       System.exit(1);
     } catch (SilentExit e) {
       System.exit(e.exitCode());
     }
   }
+
+  // Only CLI, not meant for instantiation.
+  private Main() {}
+
+  /** Print system information (e.g. in case of unexpected exceptions) */
+  public static void printSystemInfo() {
+    Out.err("Java version:     " + System.getProperty("java.version"));
+    Out.err("Runtime name:     " + System.getProperty("java.runtime.name"));
+    Out.err("Vendor:           " + System.getProperty("java.vendor"));
+    Out.err("VM version:       " + System.getProperty("java.vm.version"));
+    Out.err("VM vendor:        " + System.getProperty("java.vm.vendor"));
+    Out.err("VM name:          " + System.getProperty("java.vm.name"));
+    Out.err("VM info:          " + System.getProperty("java.vm.info"));
+    Out.err("OS name:          " + System.getProperty("os.name"));
+    Out.err("OS arch:          " + System.getProperty("os.arch"));
+    Out.err("OS version:       " + System.getProperty("os.version"));
+    Out.err("Encoding:         " + System.getProperty("file.encoding"));
+    Out.err("Unicode versions: " + UnicodeProperties.UNICODE_VERSIONS);
+    Out.err("JFlex version:    " + Build.VERSION);
+  }
 }
