@@ -11,13 +11,17 @@
 package jflex.anttask;
 
 import java.io.File;
-import java.io.FileReader;
 import java.io.IOException;
 import java.io.LineNumberReader;
+import java.io.Reader;
+import java.nio.charset.StandardCharsets;
+import java.nio.file.Files;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
-import jflex.Main;
-import jflex.Options;
+import jflex.core.OptionUtils;
+import jflex.exceptions.GeneratorException;
+import jflex.generator.LexGenerator;
+import jflex.option.Options;
 import org.apache.tools.ant.BuildException;
 import org.apache.tools.ant.Task;
 
@@ -25,7 +29,7 @@ import org.apache.tools.ant.Task;
  * JFlex ant task.
  *
  * @author Rafal Mantiuk
- * @version JFlex 1.7.0
+ * @version JFlex 1.8.2
  */
 public class JFlexTask extends Task {
   private static final Pattern PACKAGE_PATTERN = Pattern.compile("package\\s+(\\S+)\\s*;");
@@ -45,6 +49,7 @@ public class JFlexTask extends Task {
 
   /** Constructor for JFlexTask. */
   public JFlexTask() {
+    OptionUtils.setDefaultOptions();
     // ant default is different from the rest of JFlex
     setVerbose(false);
     setUnusedWarning(true);
@@ -54,9 +59,10 @@ public class JFlexTask extends Task {
   /**
    * Executes the ant task.
    *
-   * @throws org.apache.tools.ant.BuildException if any.
+   * @throws BuildException if any.
    */
-  public void execute() throws BuildException {
+  @Override
+  public void execute() {
     try {
       if (inputFile == null)
         throw new BuildException("Input file needed. Use <jflex file=\"your_scanner.flex\"/>");
@@ -69,14 +75,14 @@ public class JFlexTask extends Task {
         File destFile = new File(outputDir, className + ".java");
 
         if (inputFile.lastModified() > destFile.lastModified()) {
-          Main.generate(inputFile);
+          new LexGenerator(inputFile).generate();
           if (!Options.verbose) System.out.println("Generated: " + destFile.getName());
         }
       } catch (IOException e1) {
-        throw new BuildException("IOException: " + e1.toString());
+        throw new BuildException(e1);
       }
-    } catch (jflex.GeneratorException e) {
-      throw new BuildException("JFlex: generation failed!");
+    } catch (GeneratorException e) {
+      throw new BuildException("JFlex generation failed", e);
     }
   }
 
@@ -89,8 +95,8 @@ public class JFlexTask extends Task {
     // find name of the package and class in jflex source file
     packageName = null;
     className = null;
-
-    try (LineNumberReader reader = new LineNumberReader(new FileReader(inputFile))) {
+    Reader r = Files.newBufferedReader(inputFile.toPath(), StandardCharsets.UTF_8);
+    try (LineNumberReader reader = new LineNumberReader(r)) {
       while (className == null || packageName == null) {
         String line = reader.readLine();
         if (line == null) {
@@ -160,7 +166,7 @@ public class JFlexTask extends Task {
   }
 
   /**
-   * Getter for the field <code>className</code>.
+   * Getter for the field {@code className}.
    *
    * @return class name of input file
    * @see #findPackageAndClass()
@@ -185,7 +191,7 @@ public class JFlexTask extends Task {
    */
   public void setOutdir(File outDir) {
     this.outputDir = outDir;
-    Options.setDir(outputDir);
+    OptionUtils.setDir(outputDir);
   }
 
   /**
@@ -229,7 +235,7 @@ public class JFlexTask extends Task {
    *
    * @param verbose a boolean.
    */
-  public void setVerbose(boolean verbose) {
+  public final void setVerbose(boolean verbose) {
     Options.verbose = verbose;
     Options.unused_warning = verbose;
   }
@@ -239,7 +245,7 @@ public class JFlexTask extends Task {
    *
    * @param warn a boolean.
    */
-  public void setUnusedWarning(boolean warn) {
+  public final void setUnusedWarning(boolean warn) {
     Options.unused_warning = warn;
   }
 
@@ -249,7 +255,7 @@ public class JFlexTask extends Task {
    * @param skeleton a {@link java.io.File} object.
    */
   public void setSkeleton(File skeleton) {
-    Options.setSkeleton(skeleton);
+    OptionUtils.setSkeleton(skeleton);
   }
 
   /**
@@ -339,6 +345,6 @@ public class JFlexTask extends Task {
    * @param encodingName the name of the encoding to set (e.g. "utf-8").
    */
   public void setEncoding(String encodingName) {
-    Options.setEncoding(encodingName);
+    OptionUtils.setEncoding(encodingName);
   }
 }
