@@ -1,5 +1,5 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
- * JFlex 1.7.0                                                             *
+ * JFlex 1.8.2                                                             *
  * Copyright (C) 1998-2018  Gerwin Klein <lsf@jflex.de>                    *
  * All rights reserved.                                                    *
  *                                                                         *
@@ -10,17 +10,18 @@
 package jflex.gui;
 
 import java.io.File;
-import jflex.ErrorMessages;
-import jflex.GeneratorException;
-import jflex.Main;
-import jflex.Options;
-import jflex.Out;
+import java.util.Objects;
+import jflex.core.OptionUtils;
+import jflex.exceptions.GeneratorException;
+import jflex.generator.LexGenerator;
+import jflex.l10n.ErrorMessages;
+import jflex.logging.Out;
 
 /**
  * Low priority thread for code generation (low priority that gui has time for screen updates)
  *
  * @author Gerwin Klein
- * @version JFlex 1.7.0
+ * @version JFlex 1.8.2
  */
 public class GeneratorThread extends Thread {
 
@@ -34,7 +35,7 @@ public class GeneratorThread extends Thread {
   String outputDir;
 
   /** main UI component, likes to be notified when generator finishes */
-  MainFrame parent;
+  final MainFrame parent;
 
   /**
    * Create a new GeneratorThread, but do not run it yet.
@@ -50,6 +51,7 @@ public class GeneratorThread extends Thread {
   }
 
   /** Runs the generator thread. Only one instance of it can run at any time. */
+  @Override
   public void run() {
     if (running) {
       Out.error(ErrorMessages.ALREADY_RUNNING);
@@ -58,10 +60,10 @@ public class GeneratorThread extends Thread {
       running = true;
       setPriority(MIN_PRIORITY);
       try {
-        if (!outputDir.equals("")) {
-          Options.setDir(outputDir);
+        if (!Objects.equals(outputDir, "")) {
+          OptionUtils.setDir(outputDir);
         }
-        Main.generate(new File(inputFile));
+        new LexGenerator(new File(inputFile)).generate();
         Out.statistics();
         parent.generationFinished(true);
       } catch (GeneratorException e) {
