@@ -1,5 +1,5 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
- * JFlex 1.7.0                                                             *
+ * JFlex 1.8.2                                                             *
  * Copyright (C) 1998-2018  Gerwin Klein <lsf@jflex.de>                    *
  * All rights reserved.                                                    *
  *                                                                         *
@@ -14,12 +14,16 @@ import java.awt.Component;
  * Constraints for layout elements of GridLayout
  *
  * @author Gerwin Klein
- * @version JFlex 1.7.0
+ * @version JFlex 1.8.2
  */
 public class GridPanelConstraint {
 
-  int x, y, width, height, handle;
-  Component component;
+  final int x;
+  final int y;
+  final int width;
+  final int height;
+  final int handle;
+  final Component component;
 
   /**
    * Constructor for GridPanelConstraint.
