@@ -1,5 +1,5 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
- * JFlex 1.7.0                                                             *
+ * JFlex 1.8.2                                                             *
  * Copyright (C) 1998-2018  Gerwin Klein <lsf@jflex.de>                    *
  * All rights reserved.                                                    *
  *                                                                         *
@@ -9,7 +9,14 @@
 
 package jflex.gui;
 
-import java.awt.*;
+import java.awt.Button;
+import java.awt.Checkbox;
+import java.awt.Dialog;
+import java.awt.FileDialog;
+import java.awt.Frame;
+import java.awt.Insets;
+import java.awt.Label;
+import java.awt.TextField;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.awt.event.ItemEvent;
@@ -17,21 +24,23 @@ import java.awt.event.ItemListener;
 import java.awt.event.WindowAdapter;
 import java.awt.event.WindowEvent;
 import java.io.File;
-import jflex.GeneratorException;
-import jflex.Options;
-import jflex.Skeleton;
+import jflex.core.OptionUtils;
+import jflex.exceptions.GeneratorException;
+import jflex.option.Options;
+import jflex.skeleton.Skeleton;
 
 /**
  * A dialog for setting JFlex options
  *
  * @author Gerwin Klein
- * @version JFlex 1.7.0
+ * @version JFlex 1.8.2
  */
+@SuppressWarnings({"ConstructorInvokesOverridable"})
 public class OptionsDialog extends Dialog {
 
   private static final long serialVersionUID = 6807759416163314769L;
 
-  private Frame owner;
+  private final Frame owner;
 
   private TextField skelFile;
 
@@ -62,6 +71,7 @@ public class OptionsDialog extends Dialog {
 
     addWindowListener(
         new WindowAdapter() {
+          @Override
           public void windowClosing(WindowEvent e) {
             close();
           }
@@ -69,7 +79,7 @@ public class OptionsDialog extends Dialog {
   }
 
   /** setup. */
-  public void setup() {
+  public final void setup() {
     // create components
     Button ok = new Button("Ok");
     Button defaults = new Button("Defaults");
@@ -94,6 +104,7 @@ public class OptionsDialog extends Dialog {
     // setup interaction
     ok.addActionListener(
         new ActionListener() {
+          @Override
           public void actionPerformed(ActionEvent e) {
             close();
           }
@@ -101,6 +112,7 @@ public class OptionsDialog extends Dialog {
 
     defaults.addActionListener(
         new ActionListener() {
+          @Override
           public void actionPerformed(ActionEvent e) {
             setDefaults();
           }
@@ -108,6 +120,7 @@ public class OptionsDialog extends Dialog {
 
     skelBrowse.addActionListener(
         new ActionListener() {
+          @Override
           public void actionPerformed(ActionEvent e) {
             skelBrowse();
           }
@@ -115,6 +128,7 @@ public class OptionsDialog extends Dialog {
 
     verbose.addItemListener(
         new ItemListener() {
+          @Override
           public void itemStateChanged(ItemEvent e) {
             Options.verbose = verbose.getState();
           }
@@ -122,6 +136,7 @@ public class OptionsDialog extends Dialog {
 
     dump.addItemListener(
         new ItemListener() {
+          @Override
           public void itemStateChanged(ItemEvent e) {
             Options.dump = dump.getState();
           }
@@ -129,6 +144,7 @@ public class OptionsDialog extends Dialog {
 
     jlex.addItemListener(
         new ItemListener() {
+          @Override
           public void itemStateChanged(ItemEvent e) {
             Options.jlex = jlex.getState();
             // JLex compatibility implies that dot (.) metachar matches [^\n]
@@ -139,6 +155,7 @@ public class OptionsDialog extends Dialog {
 
     no_minimize.addItemListener(
         new ItemListener() {
+          @Override
           public void itemStateChanged(ItemEvent e) {
             Options.no_minimize = no_minimize.getState();
           }
@@ -146,6 +163,7 @@ public class OptionsDialog extends Dialog {
 
     no_backup.addItemListener(
         new ItemListener() {
+          @Override
           public void itemStateChanged(ItemEvent e) {
             Options.no_backup = no_backup.getState();
           }
@@ -153,6 +171,7 @@ public class OptionsDialog extends Dialog {
 
     dot.addItemListener(
         new ItemListener() {
+          @Override
           public void itemStateChanged(ItemEvent e) {
             Options.dot = dot.getState();
           }
@@ -160,6 +179,7 @@ public class OptionsDialog extends Dialog {
 
     legacy_dot.addItemListener(
         new ItemListener() {
+          @Override
           public void itemStateChanged(ItemEvent e) {
             Options.legacy_dot = legacy_dot.getState();
           }
@@ -167,6 +187,7 @@ public class OptionsDialog extends Dialog {
 
     time.addItemListener(
         new ItemListener() {
+          @Override
           public void itemStateChanged(ItemEvent e) {
             Options.time = time.getState();
           }
@@ -231,7 +252,7 @@ public class OptionsDialog extends Dialog {
   }
 
   private void setDefaults() {
-    Options.setDefaults();
+    OptionUtils.setDefaultOptions();
     Skeleton.readDefault();
     skelFile.setText("");
     updateState();
